{+--------------------------------------------------------------------------+
 | Copyright (c) 1998 Martin Waldenburg
 | No rights reserved.
 | DISCLAIMER:  This is provided as is, expressly without a warranty of any kind.
 |              You use it at your own risc.
 +--------------------------------------------------------------------------+}
unit Del1Edit;

interface

uses
  SysUtils, WinTypes, WinProcs, Classes, Controls, Messages, StdCtrls;

type
  TmwDel1EditHandles = class(TComponent)
  private
    fEditWindow: HWND;
    fEditControl: HWND;
    fHScroll: HWND;
    fVScroll: HWND;
    fScroller: HWND;
    fStatus: HWND;
    fTabset: HWND;
    fHelper: TCustomEdit;
    function GetTop: Integer;
    procedure SetTop(NewValue: Integer);
    function GetLineCount: Integer;
    function GetLines(Index: Integer): String;
    procedure SetLines(Index: Integer; Value: String);
    function GetWordAtCaret:String;
    procedure SetWordAtCaret(NewValue:String);
    function GetCaretLine:String;
    procedure SetCaretLine(NewValue:String);
    function GetTopLine: String;
    procedure SetTopLine(NewValue: String);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure GetEditHandles;
    function ScrollBottom: Integer;
    function EndScroll: Integer;
    function LineDown: Integer;
    function LineUp: Integer;
    function PageDown: Integer;
    function PageUp: Integer;
    function ScrollTop: Integer;
    procedure DeleteLine(Index: Integer);
    procedure InsertLine(Index: Integer; LineText: String);
    property EditControl: HWND read fEditControl;
    property EditWindow: HWND read fEditWindow;
    property HScroll: HWND read fHScroll;
    property Lines[Index: Integer]: String read GetLines write SetLines; default;
    property VScroll: HWND read fVScroll;
    property Scroller: HWND read fScroller;
    property Status: HWND read fStatus;
    property Tabset: HWND read fTabset;
    property Top: Integer read GetTop write SetTop;
    property LineCount: Integer read GetLineCount;
    property WordAtCaret:String read GetWordAtCaret write SetWordAtCaret;
    property CaretLine:String read GetCaretLine write SetCaretLine;
    property TopLine:String read GetTopLine write SetTopLine;
  published
    property Helper: TCustomEdit read fHelper write fHelper;
  end;

procedure Register;

implementation

destructor TmwDel1EditHandles.Destroy;
begin
  inherited Destroy;
end; { Destroy }

constructor TmwDel1EditHandles.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  GetEditHandles;
end; { Create }

procedure Register;
begin
  RegisterComponents('mw', [TmwDel1EditHandles]);
end; { Register }

procedure TmwDel1EditHandles.GetEditHandles;
var
  aWindow: HWnd;
  ClassN: array[0..50] of Char;
  ClassName: String;
  Position: Integer;
begin
  fEditWindow := 0;
  fEditControl := 0;
  fHScroll := 0;
  fVScroll := 0;
  fScroller := 0;
  fStatus := 0;
  fTabset := 0;
  fEditWindow := FindWindow('TEditWindow', nil);
  if fEditWindow <> 0 then
  begin
    aWindow := GetWindow(fEditWindow, GW_CHILD);
    while aWindow <> 0 do
    begin
      GetClassName(aWindow, ClassN, 20);
      Position := Pos(#0, ClassN);
      ClassName := Copy(CLassN, 1, Position - 1);
      if ClassName = 'TEditControl' then fEditControl := aWindow else
        if ClassName = 'TStatus' then fStatus := aWindow else
          if (ClassName = 'TScrollBar') and (fVScroll = 0) then fVScroll := aWindow else
            if (ClassName = 'TScrollBar') and (fVScroll <> 0) then fHScroll := aWindow else
              if ClassName = 'TTabSet' then fTabSet := aWindow else
                if ClassName = 'TScroller' then fScroller := aWindow;
      aWindow := GetWindow(aWindow, GW_HWNDNEXT);
    end;
  end;
end; { GetEditHandles }

function TmwDel1EditHandles.ScrollBottom: Integer;
var
  Pos: Integer;
  aParam: LongInt;
begin
  Pos := GetScrollPos(fVSCROLL, SB_CTL);
  aParam := MAKELONG(Pos, fVSCROLL);
  Result := SendMessage(fEditWindow, WM_VSCROLL, SB_BOTTOM, aParam);
  SendMessage(fEditControl, WM_LBUTTONDOWN, 0, 0);
  SendMessage(fEditControl, WM_LBUTTONUP, 0, 0);
end; { ScrollBottom }

function TmwDel1EditHandles.EndScroll: Integer;
var
  Pos: Integer;
  aParam: LongInt;
begin
  Pos := GetScrollPos(fVSCROLL, SB_CTL);
  aParam := MAKELONG(Pos, fVSCROLL);
  Result := SendMessage(fEditWindow, WM_VSCROLL, SB_ENDSCROLL, aParam);
end; { EndScroll }

function TmwDel1EditHandles.LineDown: Integer;
var
  Pos: Integer;
  aParam: LongInt;
begin
  Pos := GetScrollPos(fVSCROLL, SB_CTL);
  aParam := MAKELONG(Pos, fVSCROLL);
  Result := SendMessage(fEditWindow, WM_VSCROLL, SB_LINEDOWN, aParam);
end; { LineDown }

function TmwDel1EditHandles.LineUp: Integer;
var
  Pos: Integer;
  aParam: LongInt;
begin
  Pos := GetScrollPos(fVSCROLL, SB_CTL);
  aParam := MAKELONG(Pos, fVSCROLL);
  Result := SendMessage(fEditWindow, WM_VSCROLL, SB_LINEUP, aParam);
end; { LineUp }

function TmwDel1EditHandles.PageDown: Integer;
var
  Pos: Integer;
  aParam: LongInt;
begin
  Pos := GetScrollPos(fVSCROLL, SB_CTL);
  aParam := MAKELONG(Pos, fVSCROLL);
  Result := SendMessage(fEditWindow, WM_VSCROLL, SB_PAGEDOWN, aParam);
  SendMessage(fEditControl, WM_LBUTTONDOWN, 0, 0);
  SendMessage(fEditControl, WM_LBUTTONUP, 0, 0);
end; { PageDown }

function TmwDel1EditHandles.PageUp: Integer;
var
  Pos: Integer;
  aParam: LongInt;
begin
  Pos := GetScrollPos(fVSCROLL, SB_CTL);
  aParam := MAKELONG(Pos, fVSCROLL);
  Result := SendMessage(fEditWindow, WM_VSCROLL, SB_PAGEUP, aParam);
  SendMessage(fEditControl, WM_LBUTTONDOWN, 0, 0);
  SendMessage(fEditControl, WM_LBUTTONUP, 0, 0);
end; { PageUp }

function TmwDel1EditHandles.ScrollTop: Integer;
var
  Pos: Integer;
  aParam: LongInt;
begin
  Pos := GetScrollPos(fVSCROLL, SB_CTL);
  aParam := MAKELONG(Pos, fVSCROLL);
  Result := SendMessage(fEditWindow, WM_VSCROLL, SB_TOP, aParam);
  SendMessage(fEditControl, WM_LBUTTONDOWN, 0, 0);
  SendMessage(fEditControl, WM_LBUTTONUP, 0, 0);
end; { ScrollTop }

function TmwDel1EditHandles.GetLines(Index: Integer): String;
var
  aPos: LongInt;
  xPos, yPos: Integer;
begin
  Top:= Index;
  aPos:= MakeLong(0, 20);
  SendMessage(fEditControl, WM_LBUTTONDOWN, MK_SHIFT, aPos);
  SendMessage(fEditControl, WM_LBUTTONUP, 0, aPos);
  SendMessage(fEditControl, WM_Copy, 0, 0);
  fHelper.Clear;
  fHelper.PasteFromClipboard;
  SendMessage(fEditControl, WM_LBUTTONDOWN, 0, 0);
  SendMessage(fEditControl, WM_LBUTTONUP, 0, 0);
  GetLines:= fHelper.Text;
end;


procedure TmwDel1EditHandles.SetLines(Index: Integer; Value: String);
begin
  DeleteLine(Index);
  InsertLine(Index, Value);
end;


function TmwDel1EditHandles.GetTop: Integer;
begin
  SendMessage(fEditControl, WM_SETFOCUS, 0, 0);
  GetTop := GetScrollPos(fVSCROLL, SB_CTL) + 1;
end; { GetTop }

procedure TmwDel1EditHandles.SetTop(NewValue: Integer);
var
  aParam: LongInt;
begin
  SendMessage(fEditControl, WM_LBUTTONDOWN, 0, 0);
  SendMessage(fEditControl, WM_LBUTTONUP, 0, 0);
  aParam := MAKELONG(NewValue - 1, fVSCROLL);
  SendMessage(fEditWindow, WM_VSCROLL, SB_THUMBPOSITION, aParam);
  SendMessage(fEditControl, WM_LBUTTONDOWN, 0, 0);
  SendMessage(fEditControl, WM_LBUTTONUP, 0, 0);
end; { SetTop }

function TmwDel1EditHandles.GetLineCount: Integer;
var
  CurLine: LongInt;
begin
  SendMessage(fEditControl, WM_SETFOCUS, 0, 0);
  CurLine := Top;
  ScrollBottom;
  GetLineCount := Top;
  Top := CurLine;
end; { GetLineCount }

procedure TmwDel1EditHandles.DeleteLine(Index: Integer);
var
  CaretPos: TPoint;
  aPos: LongInt;
begin
  Top := Index;
  GetCaretPos(CaretPos);
  aPos := MakeLong(0, CaretPos.Y + 22); {Offset must be calculated from the font}
                                     {Unfortunately WM_GETFONT doesn't work}
                                     {The value of 22 is good up to about 15 point}
  SendMessage(fEditControl, WM_LBUTTONDOWN, MK_SHIFT, aPos);
  SendMessage(fEditControl, WM_LBUTTONUP, 0, aPos);
  SendMessage(fEditControl, WM_CLEAR, 0, 0);
end; { DeleteLine }

procedure TmwDel1EditHandles.InsertLine(Index: Integer; LineText: String);
begin
  Top := Index;
  fHelper.Clear;
  try
    fHelper.Text := LineText + #13#10;
  finally
  end;
  fHelper.SelectAll;
  fHelper.CopyToClipboard;
  SendMessage(fEditControl, WM_PASTE, 0, 0);
end; { InsertLine }

function TmwDel1EditHandles.GetWordAtCaret:String;
begin
  SendMessage(fEditControl, WM_LBUTTONDBLCLK, 100, 100);
  SendMessage(fEditControl, WM_Copy, 0, 0);
  fHelper.Clear;
  fHelper.PasteFromClipboard;
  GetWordAtCaret:= fHelper.Text;
end;   { GetWordAtCaret }

procedure TmwDel1EditHandles.SetWordAtCaret(NewValue:String);
begin
  SendMessage(fEditControl, WM_LBUTTONDBLCLK, 100, 100);
  fHelper.Text:= NewValue;
  fHelper.CopyToClipboard;
  SendMessage(fEditControl, WM_Paste, 0, 0);
end;   { SetWordAtCaret }

function TmwDel1EditHandles.GetCaretLine:String;
var
  CaretPos: TPoint;
  aPos: LongInt;
begin
  LineDown;
  SendMessage(fEditControl, WM_SETFOCUS, 0, 0);
  GetCaretPos(CaretPos);
  aPos := MakeLong(0, CaretPos.Y + 22); {Offset must be calculated from the font}
  SendMessage(fEditControl, WM_LBUTTONDOWN, MK_SHIFT, aPos);
  SendMessage(fEditControl, WM_LBUTTONUP, 0, aPos);
  SendMessage(fEditControl, WM_Copy, 0, 0);
  fHelper.Clear;
  fHelper.PasteFromClipboard;
  GetCaretLine:= fHelper.Text;
  aPos := MakeLong(CaretPos.X, CaretPos.Y);
  SendMessage(fEditControl, WM_LBUTTONDOWN, 0, aPos);
  SendMessage(fEditControl, WM_LBUTTONUP, 0, aPos);
  LineUp;
end;   { GetCaretLine }

procedure TmwDel1EditHandles.SetCaretLine(NewValue:String);
var
  CaretPos: TPoint;
  aPos: LongInt;
begin
  LineDown;
  SendMessage(fEditControl, WM_SETFOCUS, 0, 0);
  GetCaretPos(CaretPos);
  aPos := MakeLong(0, CaretPos.Y + 22); {Offset must be calculated from the font}
  SendMessage(fEditControl, WM_LBUTTONDOWN, MK_SHIFT, aPos);
  SendMessage(fEditControl, WM_LBUTTONUP, 0, aPos);
  SendMessage(fEditControl, WM_Copy, 0, 0);
  fHelper.Clear;
  fHelper.PasteFromClipboard;
  try
    fHelper.Text := NewValue + #13#10;
  finally
  end;
  fHelper.SelectAll;
  fHelper.CopyToClipboard;
  SendMessage(fEditControl, WM_PASTE, 0, 0);
  aPos := MakeLong(CaretPos.X, CaretPos.Y);
  SendMessage(fEditControl, WM_LBUTTONDOWN, 0, aPos);
  SendMessage(fEditControl, WM_LBUTTONUP, 0, aPos);
  LineUp;
end;   { SetCaretLine }

function TmwDel1EditHandles.GetTopLine: String;
begin
  GetTopLine:= Lines[Top];
end;  { GetTopLine }

procedure TmwDel1EditHandles.SetTopLine(NewValue: String);
begin
  Lines[Top]:= NewValue;
end;   { SetTopLine }

end.

