unit mwd1Navi;
{
  Vcl mwD1Navigator Expert

  Author:  Martin_Waldenburg
  Created: 20.06.98

}
{+--------------------------------------------------------------------------+
 | Copyright (c) 1998 Martin Waldenburg
 | No rights reserved.
 | DISCLAIMER:  This is provided as is, expressly without a warranty of any kind.
 |              You use it at your own risc.
 +--------------------------------------------------------------------------+}

interface

uses
  WinTypes, WinProcs, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, d1paspar, ExptIntf, ToolIntf, Del1Edit;

type
  TD1NavigatorFrm = class(TForm)
    cbDeclaration: TComboBox;
    cbImplementation: TComboBox;
    aHelper: TEdit;
    EditHandles: TmwDel1EditHandles;
    btnRefresh: TButton;
    btnReverse: TButton;
    procedure cbDeclarationClick(Sender: TObject);
    procedure cbImplementationClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnRefreshClick(Sender: TObject);
    procedure btnReverseClick(Sender: TObject);
  private
    I: Integer;
    IsClass: Boolean;
    aText: String;
    MethodKind: String;
    LIdent: String;
    LineText: String;
    ClassName: String;
    FileName: TFileName;
    Current: TComboBox;
    Parser: Td1PasParser;
    InFile: TextFile;
    Top: Integer;
    ImpLine: Integer;
    function GetCurFile: TFileName;
    procedure LoadMethods;
    procedure EvalClass;
    procedure NextLine;
    procedure MainLoad;
    procedure JumpReverse;
  protected
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  published
  end;

  TmwD1NavigatorExpert = class(TIExpert)
  public
    function GetName: string; override;
    function GetStyle: TExpertStyle; override;
    function GetIDString: string; override;
    function GetState: TExpertState; override;
    function GetMenuText: string; override;
    procedure Execute; override;
    destructor destroy; override;
  end;

procedure Register;

var
  D1NavigatorFrm: TD1NavigatorFrm;
  fToolServices: TIToolServices;

implementation

{$R *.DFM}

procedure Register;
begin
  RegisterLibraryExpert(TmwD1NavigatorExpert.Create);
end;

{ TmwD1NavigatorExpert code }

destructor TmwD1NavigatorExpert.destroy;
begin
  D1NavigatorFrm.Free;
  inherited Destroy;
end;

function TmwD1NavigatorExpert.GetName: String;
begin
  Result := 'mwD1NavigatorExpert'
end;

function TmwD1NavigatorExpert.GetStyle: TExpertStyle;
begin
  Result := esStandard;
end;

function TmwD1NavigatorExpert.GetIDString: String;
begin
  Result := 'private.mwD1NavigatorExpert';
end;

function TmwD1NavigatorExpert.GetState: TExpertState;
begin
  Result := [esEnabled]; { add more appropriate states }
end;

function TmwD1NavigatorExpert.GetMenuText: String;
begin
  Result := 'mwD1Navigator Expert...'
end;

procedure TmwD1NavigatorExpert.Execute;
begin
  if not Assigned(D1NavigatorFrm) then
    D1NavigatorFrm := TD1NavigatorFrm.Create(nil);
  D1NavigatorFrm.Show;
  D1NavigatorFrm.SetFocus
end;

{ TD1NavigatorFrm code }

function TD1NavigatorFrm.GetCurFile: TFileName;
var
  FileExt: String;
begin
  fToolServices := ExptIntf.ToolServices;
  if fToolServices.GetProjectName = '' then raise exception.create('Sorry, a project must be open');
  Result := fToolServices.GetCurrentFile;
  FileExt := ExtractFileExt(Result);
  if FileExt = '.DFM' then raise exception.create('Sorry, must be a PAS or DPR file');
end; { GetCurFile }

procedure TD1NavigatorFrm.NextLine;
begin
  readln(InFile, LineText);
  inc(I);
  Parser.Origin := LineText + #13#10#0;
end; { NextLine }

procedure TD1NavigatorFrm.EvalClass;
begin
  IsClass := False;
  ClassName:= LIdent;
  Parser.NextNonSpace;
  Case Parser.Token.ID of
    tkRoundOpen:
      begin
        IsClass := True;
        aText := aText + '(';
        while not (Parser.Token.ID = tkNull) do
        begin
          Parser.NextToken;
          if Parser.Token.ID = tkCRLF then break;
          aText := aText + Parser.Token.Data;
        end;
        Parser.NextNonJunk;
        if not (Parser.Token.ID = tkSemiColon) then
        begin
          Parser.NextNonJunk;
          if Parser.Token.ID = tkEnd then
          begin
            IsClass := False;
            while not (Parser.Token.ID = tkNull) do Parser.NextToken;
          end;
          while not (Parser.Token.ID = tkNull) do Parser.NextToken;
        end
        else
        begin
          IsClass := False;
          aText := aText + ';';
          Current.Items.AddObject(aText, Pointer(I));
        end;
        while not (Parser.Token.ID = tkNull) do Parser.NextToken;
      end;
    tkSemiColon:;
    else IsClass:= True;
  end;
  if IsClass then
  begin
    Current.Items.AddObject(aText, Pointer(I));
  end else ClassName:= '';
  while not (Parser.Token.ID = tkNull) do Parser.NextToken;
end;

procedure TD1NavigatorFrm.LoadMethods;
begin
  aText := '';
  if IsClass then aText := ClassName + ' ' + MethodKind else
  begin
    Parser.NextNonJunk;
    LIdent:= Parser.Token.Data;
    Parser.NextNonJunk;
    if Parser.Token.ID = tkPoint then
    begin
      aText := LIdent + ' ' + MethodKind + ' ';
    end else
    begin
      aText := MethodKind + ' ' + LIdent+ ' ' + Parser.Token.Data + ' ';
    end;
  end;
  while not (Parser.Token.ID = tkNull) do
  begin
    Parser.NextToken;
    if Parser.Token.ID = tkCRLF then break;
    aText := aText + Parser.Token.Data;
  end;
  Current.Items.AddObject(aText, Pointer(I));
end; { LoadMethods }

procedure TD1NavigatorFrm.cbDeclarationClick(Sender: TObject);
begin
  EditHandles.Top := Integer(cbDeclaration.Items.Objects[cbDeclaration.ItemIndex]);
end;

procedure TD1NavigatorFrm.cbImplementationClick(Sender: TObject);
begin
  EditHandles.Top := Integer(cbImplementation.Items.Objects[cbImplementation.ItemIndex]);
end;

procedure TD1NavigatorFrm.MainLoad;
begin
  cbDeclaration.Sorted:= False;
  cbImplementation.Sorted:= False;
  I := 0;
  IsClass := False;
  Current := cbDeclaration;
  Parser := Td1PasParser.Create;
  while not Eof(InFile) do
  begin
    NextLine;
    Parser.NextNonJunk;
    Case Parser.Token.ID of
      tkClass:
        begin
          MethodKind := Parser.Token.Data + ' ';
          Parser.NextNonJunk;
          Case Parser.Token.ID of
            tkFunction: MethodKind := MethodKind + Parser.Token.Data;
            tkProcedure: MethodKind := MethodKind + Parser.Token.Data;
          else MethodKind := '';
          end;
          LoadMethods;
        end;
      tkConstructor:
        begin
          MethodKind := Parser.Token.Data;
          LoadMethods;
        end;
      tkDestructor:
        begin
          MethodKind := Parser.Token.Data;
          LoadMethods;
        end;
      tkEnd:
        begin
          IsClass := False;
          ClassName:='';
          while not (Parser.Token.ID = tkNull) do Parser.NextToken;
        end;
      tkFunction:
        begin
          MethodKind := Parser.Token.Data;
          LoadMethods;
        end;
      tkIdentifier:
        begin
          LIdent := Parser.Token.Data;
          Parser.NextNonJunk;
          if Parser.Token.ID = tkEqual then
          begin
            Parser.NextNonJunk;
            if Parser.Token.ID = tkClass then
            begin
              aText := LIdent + ' = ' + Parser.Token.Data;
              EvalClass;
            end;
          end;
        end;
      tkImplementation:
        begin
          Current := cbImplementation;
          ImpLine:= I;
        end;
      tkProcedure:
        begin
          MethodKind := Parser.Token.Data;
          LoadMethods;
        end;
      tkProperty:
        begin
          MethodKind := Parser.Token.Data;
          LoadMethods;
        end;
    else while not (Parser.Token.ID = tkNull) do Parser.NextToken;
    end;
  end;
  Parser.Free;
  cbDeclaration.Sorted:= True;
  cbImplementation.Sorted:= True;
end;

procedure TD1NavigatorFrm.JumpReverse;
var
  ItemText, SearchText: String;
  I: Integer;
  FoundText, FoundIndex: Boolean;
begin
  FoundText:= False;
  FoundIndex:= False;
  Parser := Td1PasParser.Create;
  if Top < ImpLine then
  begin
    for I:= 0 to cbDeclaration.Items.Count -1 do
    begin
      if Integer(cbDeclaration.Items.Objects[I]) = Top then
      begin
        cbDeclaration.ItemIndex:= I;
        FoundIndex:= True;
        break;
      end;
    end;
    if FoundIndex then
    begin
      ItemText:= cbDeclaration.Items[I];
      Parser.Origin := ItemText + #13#10#0;
      SearchText:= '';
      for I:= 1 to 5 do
      begin
        Parser.NextToken;
        SearchText:= SearchText + Parser.Token.Data;
      end;
      for I:= 0 to cbImplementation.Items.Count -1 do
      begin
        ItemText:= cbImplementation.Items[I];
        if Pos(SearchText, ItemText) = 1 then
        begin
          FoundText:= True;
          break;
        end;
      end;
      if FoundText then
      begin
        EditHandles.Top := Integer(cbImplementation.Items.Objects[I]);
        Application.ProcessMessages;
      end;
    end;
  end else
  begin
    for I:= 0 to cbImplementation.Items.Count -1 do
    begin
      if Integer(cbImplementation.Items.Objects[I]) = Top then
      begin
        cbImplementation.ItemIndex:= I;
        FoundIndex:= True;
        break;
      end;
    end;
    if FoundIndex then
    begin
      ItemText:= cbImplementation.Items[I];
      Parser.Origin := ItemText + #13#10#0;
      SearchText:= '';
      for I:= 1 to 5 do
      begin
        Parser.NextToken;
        SearchText:= SearchText + Parser.Token.Data;
      end;
      for I:= 0 to cbDeclaration.Items.Count -1 do
      begin
        ItemText:= cbDeclaration.Items[I];
        if Pos(SearchText, ItemText) = 1 then
        begin
          FoundText:= True;
          break;
        end;
      end;
      if FoundText then
      begin
        EditHandles.Top := Integer(cbDeclaration.Items.Objects[I]);
        Application.ProcessMessages;
      end;
    end;
  end;
  Parser.Free;
end;

procedure TD1NavigatorFrm.FormShow(Sender: TObject);
var
  Temp: array[0..80] of char;
begin
  EditHandles.GetEditHandles;
  {SendMessage(EditHandles.EditWindow, WM_GETTEXT, SizeOf(Temp), LongInt(@Temp));
  FileName := StrPas(Temp);}
  FileName:= GetCurFile;
  D1NavigatorFrm.Caption:= FileName;
  AssignFile(InFile, FileName);
  reset(InFile);
  MainLoad;
  CloseFile(InFile);
end;

procedure TD1NavigatorFrm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  cbDeclaration.Clear;
  cbImplementation.Clear;
end;

procedure TD1NavigatorFrm.btnRefreshClick(Sender: TObject);
begin
  cbDeclaration.Clear;
  cbImplementation.Clear;
  EditHandles.GetEditHandles;
  FileName:= GetCurFile;
  D1NavigatorFrm.Caption:= FileName;
  AssignFile(InFile, FileName);
  reset(InFile);
  MainLoad;
  CloseFile(InFile);
end;

procedure TD1NavigatorFrm.btnReverseClick(Sender: TObject);
begin
  Top:= EditHandles.Top;
  Application.ProcessMessages;
  JumpReverse;
end;

end.

